@@ -28,14 +28,41 @@
 #' json <- tmp %>% ec.inspect()
 #' ec.fromJson(json) %>% ec.theme("dark")
 #'
+#'
 #' #------ Data grouping
-#' iris %>% dplyr::group_by(Species) %>% ec.init()     # by factor column
-#' p <- Orange %>% dplyr::group_by(Tree) %>% ec.init() # no factor column
-#' p$x$opts$series[[1]] <- append(p$x$opts$series[[1]], list(
-#'   symbolSize = 10
-#' ))   # further customization added
+#' library(dplyr)
+#' iris %>% group_by(Species) %>% ec.init()       # by factor column
+#' iris %>% mutate(Species=as.character(Species)) %>%
+#'          group_by(Species) %>% ec.init()       # by non-factor column
+#' 
+#' p <- Orange %>% group_by(Tree) %>% ec.init()   # by factor column
+#' p$x$opts$series <- lapply(p$x$opts$series, function(x) { 
+#'   x$symbolSize=10; x$encode=list(x='age', y='circumference'); x } )
 #' p
 #'
+#'
+#' #------ Pie
+#' is <- sort(islands); is <- is[is>60]
+#' is <- data.frame(name=names(is), value=as.character(unname(is)))
+#' data <- apply(unname(is), 1, function(x) list(name=x[1], value=x[2]))
+#' p <- ec.init()
+#' p$x$opts <- list(
+#'   title = list(text = "Landmasses over 60,000 mi\u00B2", left = 'center'),
+#'   tooltip = list(trigger='item'),
+#'   series = list(type='pie', radius='50%', data=data, name='mi\u00B2',
+#'     emphasis=list(itemStyle=list(shadowBlur=10, shadowColor='rgba(0,0,0,0.5)'))))
+#' p
+#' 
+#' #------ Liquidfill plugin
+#' if (interactive()) {
+#' p <- ec.init(load=c('liquid'), preset=FALSE)
+#' p$x$opts$series[[1]] <- list(
+#'   type='liquidFill', data=c(0.6, 0.5, 0.4, 0.3), # amplitude=0,
+#'   waveAnimation=FALSE, animationDuration=0, animationDurationUpdate=0
+#' )
+#' p
+#' }
+#' 
 #' #------ Heatmap
 #' times <- c(5,1,0,0,0,0,0,0,0,0,0,2,4,1,1,3,4,6,4,4,3,3,2,5,7,0,0,0,0,0,
 #'            0,0,0,0,5,2,2,6,9,11,6,7,8,12,5,5,7,2,1,1,0,0,0,0,0,0,0,0,3,2,
@@ -67,30 +94,34 @@
 #' p$x$opts$series[[1]]$symbolSize = htmlwidgets::JS("function(x){ return x[3];}")
 #' p
 #'  
-#' #------ Plugins 3D and GL
-#' p <- ec.init(load = c('3D','GL'))
-#' p$x$opts$series[[1]] <- list(
+#' #------ Plugin 3D
+#' if (interactive()) {
+#' p <- ec.init(load = '3D')
+#' p$x$opts$series <- list(
 #'   type = 'surface',
-#'   data = ec.data(as.data.frame(as.table(volcano)), TRUE)
+#'   data = ec.data(as.data.frame(as.table(volcano)), 'values')  
 #' )
-#' p 
+#' p
+#' }
 #' 
 #' #------ 3D chart with custom coloring
-#' #  [4] is the JS index of column Species
-#' p <- iris %>% ec.init(load = c('3D','GL'))
+#' if (interactive()) {
+#' p <- iris %>% ec.init(load = '3D')
 #' p$x$opts$series[[1]] <- list(
 #'   type='scatter3D', symbolSize=7, 
 #'   encode=list(x='Sepal.Length', y='Sepal.Width', z='Petal.Length'),
 #'   itemStyle=list(color = htmlwidgets::JS("function(params){
-#'       if (params.value[4] == 1)    { return '#FE8463'; }
-#'       else if(params.value[4] == 2){ return '#27727B'; }
-#'       return '#9BCA63';
-#'   }") )
+#'           if (params.value[4] == 1){ return '#FE8463'; }
+#'      else if (params.value[4] == 2){ return '#27727B'; }
+#'                                      return '#9BCA63';
+#'   }") )  # [4] is the JS index of column Species
 #' )
 #' p
+#' }
 #' 
 #' #------ Surface data equation with JS code
-#' p <- ec.init(load=c('3D','GL'))
+#' if (interactive()) {
+#' p <- ec.init(load='3D')
 #' p$x$opts$series[[1]] <- list(
 #'   type = 'surface',
 #'   equation = list(
@@ -101,41 +132,47 @@
 #'   )
 #' )
 #' p
+#' }
 #'
 #' #------ Surface with data from a data.frame
+#' if (interactive()) {
 #' library(dplyr)
 #' data <- expand.grid(
 #'   x = seq(0, 2, by = 0.1),
 #'   y = seq(0, 1, by = 0.1)
 #' ) %>% mutate(z = x * (y ^ 2)) %>% select(x,y,z)
-#' p <- ec.init(load=c('3D','GL'))
+#' p <- ec.init(load='3D')
 #' p$x$opts$series[[1]] <- list(
 #'   type = 'surface',
-#'   data = ec.data(data, TRUE))
+#'   data = ec.data(data, 'values'))
 #' p
+#' }
 #' 
 #' #------ Band serie with customization
-#' # Custom plugin for ec.sband's renderer
-#' # initial or later data loading is shown (switch commented lines)
-#' # first column usually sets the X-axis
+#' # first column ('day') usually goes to the X-axis
+#' # try also alternative data setting - replace lines @1 & @2 with
+#' #   @1: p <- dats %>% ec.init(load='custom') 
+#' #   @2: encode=list(x='day', y='CAC')
 #' library(dplyr)
-#' dats <- as.data.frame(EuStockMarkets) %>% mutate(day=1:n()) %>% 
+#' dats <- as.data.frame(EuStockMarkets) %>% mutate(day=1:n()) %>%
 #'   relocate(day) %>% slice_head(n=200)
-#' # p <- dats %>% ec.init(load='custom') # needs encode below
-#' p <- ec.init(load='custom')            # need data below
-#' p$x$opts$series[[1]] <- ec.sband(dats, 'DAX','FTSE')
-#' p$x$opts$series[[1]] <- append(p$x$opts$series[[1]], list(
-#'   name = 'band', color = 'lemonchiffon'
-#' ))
-#' p$x$opts$series[[2]] <- list(
-#'   type='line', name='CAC', color='red', symbolSize=1,
-#'   # encode=list(x='day', y='CAC'),
-#'   data = ec.data(dats %>% select(day,CAC), TRUE) )
-#' p$x$opts$legend = list(data=list(
-#'   list(name='band'), list(name='CAC') ))
-#' p$x$opts$dataZoom <- list(list(type='slider',start=50))
+#' p <- ec.init(load='custom')            # @1
+#' p$x$opts <- list(
+#'   xAxis = list(list()),
+#'   yAxis = list(list()),
+#'   series = list(
+#'     append( ecr.band(dats, 'DAX','FTSE'), list(
+#'       name='band', color='lemonchiffon')),  # band + customize
+#'     list(type='line', name='CAC', color='red', symbolSize=1,
+#'          data = ec.data(dats %>% select(day,CAC), 'values') )  # @2
+#'   ),
+#'   legend = list(data=list(
+#'     list(name='band'), list(name='CAC') )),
+#'   dataZoom = list(list(type='slider',start=50))
+#' )
 #' p
 #' 
+#' 
 #' #------ Timeline animation
 #' orng <- Orange
 #' orng$Tree <- paste('tree',as.character(orng$Tree)) # to string
@@ -168,6 +205,7 @@
 #' p$x$opts$options <- options
 #' p
 #' 
+#' 
 #' #------ Groups in a boxplot
 #' # original JS: https://echarts.apache.org/examples/en/editor.html?c=boxplot-multi
 #' grps <- list()   # data in 3 groups
@@ -206,6 +244,7 @@
 #' p$x$opts$dataZoom <- list(list(type='slider',start=50)) 
 #' p
 #' 
+#' 
 #' #------ EChartsJS v.5 feature custom transform - a regression line
 #' # presets for xAxis,yAxis,dataset and series are used
 #' dset <- data.frame(x=1:10, y=sample(1:100,10))
@@ -261,6 +300,37 @@
 #' p$x$opts$tooltip <- list(trigger='item', axisPointer=list(type='shadow'))
 #' p
 #'
+#'
+#' #------ Sunburst
+#' data = list(list(name='Grandpa',children=list(list(name='Uncle Leo',value=15,
+#'      children=list(list(name='Cousin Jack',value=2), list(name='Cousin Mary',value=5,
+#'      children=list(list(name='Jackson',value=2))), list(name='Cousin Ben',value=4))), 
+#'    list(name='Father',value=10,children=list(list(name='Me',value=5), 
+#'    list(name='Brother Peter',value=1))))), list(name='Nancy',children=list(
+#'    list(name='Uncle Nike',children=list(list(name='Cousin Betty',value=1), 
+#'    list(name='Cousin Jenny',value=2))))))
+#' p <- ec.init()
+#' p$x$opts <- list(
+#'   series = list(list(type='sunburst', data=data, radius=list(0, '90%'),label=list(rotate='radial')))
+#' )
+#' p
+#' 
+#' 
+#' #------ registerMap JSON
+#' json <- jsonlite::read_json("https://echarts.apache.org/examples/data/asset/geo/USA.json")
+#' dusa <- USArrests %>% dplyr::mutate(states = row.names(.))
+#' p <- ec.init(preset=FALSE)
+#' p$x$registerMap <- list(list(mapName = 'USA', geoJSON = json))
+#' p$x$opts <- list(
+#'   visualMap = list(type='continuous', calculable=TRUE, 
+#'                    min=min(dusa$UrbanPop), max=max(dusa$UrbanPop))
+#'   ,series = list( list(type='map', map='USA', name='UrbanPop', roam=TRUE,
+#'        data = lapply(ec.data(dusa,'names'), function(x) list(name=x$states, value=x$UrbanPop))
+#'    ))
+#' )
+#' p  
+#' 
+#' 
 #' #------ Gauge
 #' p <- ec.init(preset = FALSE); 
 #' p$x$opts$series[[1]] <- list( 
@@ -269,6 +339,7 @@
 #'   data = list(list(value=85, name='IQ test')) )
 #' p
 #' 
+#' 
 #' #------ Custom gauge with animation
 #' p <- ec.init(js = "setInterval(function () { 
 #'    opts.series[0].data[0].value = (Math.random() * 100).toFixed(2) - 0;  
@@ -298,11 +369,8 @@
 #' 
 #' p <- ec.init(preset=FALSE)
 #' p$x$opts$series[[1]] <- list( type='sankey',
-#'   data = lapply(ec.data(sankey,TRUE),
-#'                 function(x) list(name=x$value[1])),
-#'   edges = lapply(ec.data(sankey,TRUE), function(x)
-#'     list(source=as.character(x$value[2]), 
-#'          target=as.character(x$value[3]), value=x$value[4]) ) 
+#'   data = lapply(ec.data(sankey,'names'), function(x) list(name=x$node)),
+#'   edges = ec.data(sankey,'names')
 #' )
 #' p
 #' 
@@ -310,12 +378,10 @@
 #' p <- ec.init(preset=FALSE, title=list(text="Graph"))
 #' p$x$opts$series[[1]] <- list( type='graph',
 #'   layout = 'force',   # try 'circular' too
-#'   data = lapply(ec.data(sankey,TRUE),
-#'                 function(x) list(name=x$value[1], tooltip = list(show=FALSE))),
-#'   edges = lapply(ec.data(sankey,TRUE), 
-#'                 function(x) list(source=x$value[2], 
-#'                   target=x$value[3], value=x$value[4], 
-#'                   lineStyle = list(width=x$value[4]))), 
+#'   data = lapply(ec.data(sankey,'names'),
+#'              function(x) list(name=x$node, tooltip = list(show=FALSE))),
+#'   edges = lapply(ec.data(sankey,'names'),
+#'              function(x) { x$lineStyle <- list(width=x$value); x }),
 #'   emphasis = list(focus='adjacency',
 #'                   label=list( position='right', show=TRUE)),
 #'   label = list(show=TRUE), roam = TRUE, zoom = 4,
@@ -328,34 +394,48 @@
 #'    
 #' #------ Shiny interactive chart
 #' if (interactive()) {
-#' library(shiny)
-#' library(dplyr)
+#'   library(shiny)
+#'   library(dplyr)
 #' runApp( list(
 #'   ui = fluidPage(
 #'     ecs.output('plot'),
-#'     actionButton('addm', 'Add marks'),
-#'     actionButton('delm', 'Del area+line marks'), HTML('&nbsp; &nbsp; &nbsp; &nbsp;'),
-#'     actionButton('adds', 'Add serie'),
-#'     actionButton('dels', 'Del serie'), HTML('&nbsp; &nbsp; &nbsp; &nbsp;'),
-#'     actionButton('hilit', 'Highlight'),
-#'     actionButton('dnplay', 'Downplay')
+#'     fluidRow( 
+#'       column(4, actionButton('addm', 'Add marks'),
+#'              actionButton('delm', 'Delete marks'),
+#'              br(),span('mark points stay, area/line deletable')
+#'       ),
+#'       column(3, actionButton('adds', 'Add serie'),
+#'              actionButton('dels', 'Del serie')), 
+#'       column(5, actionButton('adata', 'Add data'),
+#'              actionButton('hilit', 'Highlight'),
+#'              actionButton('dnplay', 'Downplay') )
+#'     )
 #'   ),
 #'   server = function(input, output, session){
-#' 
+#'     
 #'     output$plot <- ecs.render({
-#'       p <- mtcars %>% group_by(cyl) %>% ec.init()
+#'       p <- ec.init()
+#'       p$x$opts$series <- lapply(mtcars %>% 
+#'         relocate(disp, .after=mpg) %>% group_by(cyl) %>% group_split(), 
+#'         function(s) { list(type='scatter', 
+#'         name=unique(s$cyl), data=ec.data(s, 'values')) })
+#'       p$x$opts$legend <- list(ey='')
+#'       p$x$opts$xAxis <- list(type="value"); 
+#'       p$x$opts$yAxis <- list(ec='')
 #'       p$x$opts$tooltip <- list(list(show=TRUE))
-#'       p$x$opts$series[[1]]$emphasis <- list(focus='series', blurScope='coordinateSystem')
+#'       p$x$opts$series[[1]]$emphasis <- list(
+#'         focus='series', blurScope='coordinateSystem')
 #'       p
 #'     })
-#' 
+#'     
 #'     observeEvent(input$addm, {
 #'       p <- ecs.proxy('plot')
-#'       p$x$opts$series[[1]] = list(
+#'       p$x$opts$series = list( list(
 #'         markPoint = list(data = list(
 #'           list(coord = c(22.5, 140.8)),
 #'           list(coord = c(30.5, 95.1))
-#'         ), itemStyle = list(color='lightblue')
+#'         ), 
+#'         itemStyle = list(color='lightblue')
 #'         )
 #'         ,markArea = list(data = list(list(
 #'           list(xAxis = 15),
@@ -366,21 +446,42 @@
 #'         ,label = list(formatter='X-area', position='insideTop')
 #'         )
 #'         ,markLine = list(data = list(list(type='average')))
-#'       )
-#'       p %>% ecs.exec() # ='p_merge'
+#'       ), list(
+#'         markPoint = list(data = list(
+#'           list(coord = c(25.5, 143.8)),
+#'           list(coord = c(33.5, 98.1))
+#'         ), 
+#'         itemStyle = list(color='forestgreen')
+#'         )
+#'       ))
+#'       p %>% ecs.exec() #' ='p_merge'
 #'     })
+#'     
 #'     observeEvent(input$adds, {
 #'       p <- ecs.proxy('plot')
-#'       p$x$opts$series[[1]] <- list(
+#'       p$x$opts$series <- list(list(
 #'         type = 'line', name = 'newLine',
-#'         encode = list(x='mpg', y='disp')
-#'       )
+#'         #encode = list(x='mpg', y='disp')  # for dataset only
+#'         data = list(list(10,100),list(5,200),list(10,400),
+#'                     list(10,200),list(15,150),list(5,300))
+#'       ))
 #'       p %>% ecs.exec('p_update')
 #'     })
+#'     
+#'     observeEvent(input$adata, {
+#'       set.seed(sample(1:444, 1))
+#'       tmp <- apply(unname(data.frame(rnorm(5, 10, 3), rnorm(5, 200, 33))), 
+#'                    1, function(x) { list(value=x) })
+#'       p <- ecs.proxy('plot')
+#'       p$x$opts$seriesIndex <- 1
+#'       p$x$opts$data <- tmp
+#'       p %>% ecs.exec('p_append_data')
+#'     })
+#'     
 #'     observeEvent(input$dels, {
 #'       p <- ecs.proxy('plot')
 #'       p$x$opts$seriesName <- 'newLine'
-#'       # p$x$opts$seriesIndex <- 4  # alternative ok
+#'       #'p$x$opts$seriesIndex <- 4  # ok too
 #'       p %>% ecs.exec('p_del_serie')
 #'     })
 #'     observeEvent(input$delm, {
@@ -399,14 +500,13 @@
 #'       p$x$opts <- list(type='downplay', seriesName='4')
 #'       p %>% ecs.exec('p_dispatch')
 #'     })
-#'   }
-#' ))
+#'   } ))
 #' }
 #' 
 #' }
 #' 
 #' @export 
 ec.examples <- function(){
-  cat('copy/paste code from ?ec.examples Help')
+  cat("copy/paste code from ?ec.examples Help\n Or run all examples at once with example('ec.examples') and they'll show up in the Viewer.")
 }
 
